// -*- C++ -*-
/*!
 * @file  kineTest.cpp
 * @brief RH1 kinematic test module
 * @date $Date$
 *
 * $Id$
 */

#include "kineTest.h"

#include "RH1Controller/kineRH1.h"

// Module specification
// <rtc-template block="module_spec">
static const char* kinetest_spec[] =
  {
    "implementation_id", "kineTest",
    "type_name",         "kineTest",
    "description",       "RH1 kinematic test module",
    "version",           "1.0.0",
    "vendor",            "AIST",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
kineTest::kineTest(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inDataIn("inData", m_inData),
    m_outHandPosOut("outHandPos", m_outHandPos),

    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("inData", m_inDataIn);
  
  // Set OutPort buffer
  registerOutPort("outHandPos", m_outHandPosOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
kineTest::~kineTest()
{
}


/*
RTC::ReturnCode_t kineTest::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t kineTest::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t kineTest::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t kineTest::onExecute(RTC::UniqueId ec_id)
{
  std::cout << std::endl;
  std::cout << "Command list: " << std::endl;
  std::cout << " rh1ik (IK test with hand pos and ori) " << std::endl;
  std::cout << " quit" << std::endl;
  std::cout << "> ";

  std::string args;
  std::string::size_type pos;
  std::vector<std::string> argv;
  std::getline(std::cin, args);

  if (args.length() < 3)
    return RTC::RTC_OK;

  pos = args.find_first_of(" ");
  if (pos != std::string::npos)
    {
      argv.push_back(args.substr(0, pos));
      argv.push_back(args.substr(++pos));
    }
  else
    {
      argv.push_back(args);
    }

  if (argv[0] == "rh1ik"){
    std::vector<double> ref_pos(6);
    std::cout << "Please input reference x[mm], y[mm], z[mm], rx[deg], ry[deg], rz[deg]: "
	      <<std::endl;
    std::cin >> ref_pos[0]>> ref_pos[1]>> ref_pos[2]>> ref_pos[3]>> ref_pos[4]>>ref_pos[5];

    std::cout<<" ref_pos ori ";
    for(unsigned int i=0; i<3; i++){
      std::cout<<" "<<ref_pos[3+i];
    }
    std::cout<<std::endl;

    vector3 pos;
    matrix33 rot;

    pos = vector3(ref_pos[0], ref_pos[1], ref_pos[2]);
    rot = OpenHRP::rotFromRpy(deg2rad(ref_pos[3]), deg2rad(ref_pos[4]),
			      deg2rad(ref_pos[5]));

    std::cout<<"rot: "<<rot<<std::endl;

    // get homogeneous matrix
    m_outHandPos.data.length(16);
    for(unsigned int i=0; i<3; i++){
      for(unsigned int j=0; j<3; j++)
	m_outHandPos.data[i*4+j] = rot(i, j);
      m_outHandPos.data[i*4+3] = pos(i); 
    }
    for(unsigned int i=0; i<3; i++)
      m_outHandPos.data[12+i]=0.0;
    m_outHandPos.data[15]=1.0;

    std::cout<<"computed homogenous matrix: size "<<m_outHandPos.data.length()<<std::endl;
    for(unsigned int i=0; i<4; i++){
      for(unsigned int j=0; j<4; j++)
	std::cout<<m_outHandPos.data[i*4+j]<<" ";
      std::cout<<std::endl;
    }

    
    m_outHandPosOut.write();
  }

  else if (argv[0] == "quit" || argv[0] == "exit")
    std::exit(1);

  if(m_inDataIn.isNew()){
    m_inDataIn.read();

    std::cout<<"data received: size "<<m_inData.data.length()<<std::endl;
    std::cout<<" data read: "<<std::endl;
    for(unsigned int i=0; i<m_inData.data.length(); i++){
      std::cout<<" "<<m_inData.data[i];
    }
    std::cout<<std::endl;

    std::cout<<" In deg : "<<std::endl;
    for(unsigned int i=0; i<m_inData.data.length(); i++){
      std::cout<<" "<<rad2deg(m_inData.data[i]);
    }
    std::cout<<std::endl;
      
  }

  return RTC::RTC_OK;

}


/*
RTC::ReturnCode_t kineTest::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t kineTest::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void kineTestInit(RTC::Manager* manager)
  {
    RTC::Properties profile(kinetest_spec);
    manager->registerFactory(profile,
                             RTC::Create<kineTest>,
                             RTC::Delete<kineTest>);
  }
  
};


