#ifndef HPP_RTM_TP_TOOLS_H
#define HPP_RTM_TP_TOOLS_H

#include <rtm/idl/BasicDataTypeSkel.h>

#include "PlannerData.hh"
#include <vector>

#define MIN_DIST 1e-6

/**
    \brief calculate minimum jerk trajectory
    \param i_traj sparse trajectory input
    \param i_samplingTime sampling time
    \param i_uvlimit velocity upper limit of the angle
    \param i_lvlimit velocity lower limit of the angle
    \return o_traj output trajectory (vector [time][joint_id])
*/
bool calcMinJerkTrajectory(const TrajData &i_traj, double i_samplingTime, 
			   const std::vector<double> &i_uvlimit, 
			   const std::vector<double> &lvlimit, 
			   std::vector<std::vector<double> > &o_traj);

/**
    \brief calculate minimum jerk trajectory
    \param i_traj sparse trajectory input
    \param i_samplingTime sampling time
    \param i_uvlimit velocity upper limit of the angle
    \param i_lvlimit velocity lower limit of the angle
    \return o_traj output trajectory
*/
bool calcMinJerkTrajectory(const TrajData &i_traj, double i_samplingTime, 
			   const std::vector<double> &i_uvlimit, 
			   const std::vector<double> &lvlimit, 
			   TrajData &o_traj);

/**
   \brief calculate mimimum jerk trajectory for an path element
   \param 
*/
bool calcExecTime(const RTC::TimedDoubleSeq &i_cfg_init, const RTC::TimedDoubleSeq &i_cfg_final,
		  double i_samplingTime,
		  const std::vector<double> &i_uvlimit, const std::vector<double> &lvlimit,
		  double &o_time);

/**
   \brief calculate the element minimum-jerk trajectory and add to o_trajVec.
*/
bool calcMinJerkElemTraj(const RTC::TimedDoubleSeq &i_cfg_init, const RTC::TimedDoubleSeq &i_cfg_final,
			 double i_time_traj, double i_samplingTime, 
			 std::vector<std::vector<double> > &o_trajVec);

/**
   \brief computation of mininum jerk trajectory
*/
void minJerkInterpolation(double tot_time, double x_init, double x_fin, double samplingTime, 
			  std::vector<double> &seq_x);

/**
   \brief internal computation for minimum jerk trajectory
*/
bool privateMinJerkTrajectory(const TrajData &i_traj, double i_samplingTime, 
			      const std::vector<double> &i_uvlimit, 
			      const std::vector<double> &i_lvlimit, 
			      std::vector<std::vector<double> > &o_trajVec);

/**
   \brief print TrajData
*/
void printPath(std::ostream &outs, const TrajData &path);

/**
   \brief convert from vector to TrajData
*/
void vec2TrajData(const std::vector<std::vector<double> > &i_vec, TrajData &o_traj, 
		  double i_samplingTime);

/**
   \brief convert from TrajData to vector
*/
void TrajData2vec(const TrajData &i_traj, std::vector<std::vector<double> > &o_vec);

/**
   \brief get date and time
*/
void getStrDateTime(char* fname_time);

/**
   \brief get data from string including values delimited by delimitor
*/
std::vector<double> getVectorDelim(const std::string &i_line, 
				   const std::string &i_delim);

#endif
